@@ -28,6 +28,7 @@
 #endif
 
 #include "accessibility.h"
+#include "keyboardfocusindication.h"
 #include "pinentryconfirm.h"
 #include "pinentrydialog.h"
 #include "pinentry.h"
@@ -71,6 +72,10 @@
 #include <windows.h>
 #endif
 
+#if PINENTRY_KWINDOWSYSTEM
+#include <KWindowSystem>
+#endif
+
 #include "pinentry_debug.h"
 
 static QString escape_accel(const QString &s)
@@ -170,6 +175,16 @@ setup_foreground_window(QWidget *widget, WId parentWid)
                            Qt::WindowMinimizeButtonHint);
 }
 
+static void
+setup_foreground_window(QWidget *widget, const QString &parentWid)
+{
+#if PINENTRY_KWINDOWSYSTEM
+    widget->winId(); // Important; ensures that a window handle is returned.
+    KWindowSystem::setMainWindow(widget->windowHandle(), parentWid);
+#endif
+}
+
+
 static int
 qt_cmd_handler(pinentry_t pe)
 {
@@ -211,13 +226,20 @@ qt_cmd_handler(pinentry_t pe)
     const QString generateTT = pe->genpin_tt ? from_utf8(pe->genpin_tt) :
                                QString();
 
+    const QString savePassphraseText =
+        pe->default_pwmngr ? escape_accel(from_utf8(pe->default_pwmngr)) :
+        QStringLiteral("Save passphrase in password manager");
 
     if (want_pass) {
-        PinEntryDialog pinentry(nullptr, 0, pe->timeout, true, !!pe->quality_bar,
+        PinEntryDialog pinentry(pe, nullptr, 0, true,
                                 repeatString, visibilityTT, hideTT);
-        setup_foreground_window(&pinentry, pe->parent_wid);
-        pinentry.setPinentryInfo(pe);
+        if (qApp->platformName() == QStringLiteral("wayland")) {
+            setup_foreground_window(&pinentry, QUrl::fromPercentEncoding(qgetenv("PINENTRY_GEOM_HINT").split(' ')[0]));
+        } else {
+            setup_foreground_window(&pinentry, pe->parent_wid);
+        }
         pinentry.setPrompt(escape_accel(from_utf8(pe->prompt)));
+
         pinentry.setDescription(from_utf8(pe->description));
         pinentry.setRepeatErrorText(repeatError);
         pinentry.setGenpinLabel(generateLbl);
@@ -232,6 +254,7 @@ qt_cmd_handler(pinentry_t pe)
             from_utf8(pe->constraints_hint_long),
             from_utf8(pe->constraints_error_title)
         });
+        pinentry.setSavePassphraseCBText(savePassphraseText);
 
         if (!title.isEmpty()) {
             pinentry.setWindowTitle(title);
@@ -290,7 +313,11 @@ qt_cmd_handler(pinentry_t pe)
         box.setTextFormat(Qt::PlainText);
         box.setTextInteractionFlags(Qt::TextSelectableByMouse);
         box.setTimeout(std::chrono::seconds{pe->timeout});
-        setup_foreground_window(&box, pe->parent_wid);
+        if (qApp->platformName() == QStringLiteral("wayland")) {
+            setup_foreground_window(&box, QUrl::fromPercentEncoding(qgetenv("PINENTRY_GEOM_HINT").split(' ')[0]));
+        } else {
+            setup_foreground_window(&box, pe->parent_wid);
+        }
 
         const struct {
             QMessageBox::StandardButton button;
@@ -418,7 +445,9 @@ main(int argc, char *argv[])
         new_argc = argc;
         Q_ASSERT (new_argc);
         app = new QApplication(new_argc, new_argv);
-        app->setWindowIcon(QIcon(QLatin1String(":/icons/document-encrypt.png")));
+        app->setWindowIcon(QIcon(QLatin1String(":/icons/pinentry.png")));
+        app->setDesktopFileName(QStringLiteral("org.gnupg.pinentry-qt"));
+        (void) new KeyboardFocusIndication{app};
     }
 
     pinentry_parse_opts(argc, argv);
